include  "$scripts\Resources\CombineDetachSkin.ms"
include  "$scripts\Resources\Backfaces.ms"

--QoL functions
fn MeshToPoly a =
(
    max modify mode
    select a
    addModifier a (turn_to_poly()) before:a.modifiers.count	
    maxOps.CollapseNodeTo a a.modifiers.count false
)

fn SkinOnTop obj =
(
    clearlistener()
    max modify mode
    
    --INIT SOME VARIABLES
    local selObjs = getCurrentSelection()
    local selCount = selObjs.count
    
    --BONENAMEARRAY
    local NameArray = #()

    --SAVE WEIGHTS
    local createms = "" as stringstream
    --local totalVerts = 0
    
    modPanel.setCurrentObject obj.skin
    
    if InstanceMgr.CanMakeModifiersUnique obj obj.skin do
    (
        InstanceMgr.MakeModifiersUnique obj obj.skin #individual
        invalidate obj.skin
    )
    
    local sk = obj.skin
    modPanel.setCurrentObject sk
    
    local bCount = GetNumberBones sk
    
    for b = 1 to bCount do
    (
        appendifUnique NameArray (GetBoneName sk b 1)
    )
    
    sort NameArray
    
    skinCount = GetNumberVertices sk
    
    sk.clearZeroLimit = 0.0
    RemoveZeroWeights sk
    
    for v = 1 to skinCount do
    (
        weights = #()
        _bones = #()
        
        weightCount = GetVertexWeightCount sk v

        for b = 1 to weightCount do
        (
            weight = GetVertexWeight sk v b
            _boneID = GetVertexWeightBoneID sk v b
            _bone = GetBoneName sk _boneID 1
            --_bone = "$'" + _bone + "'"	
                
            if weight != 0 do
            (
                append weights weight
                append _bones _bone
            )
        )
        
        format "%\n%\n%\n" v weights _bones to:createms
    )
    
    --Delete old skin mod
    deleteModifier obj obj.modifiers[#Skin]
    
    --Add new skin mod to put it at the top of the stack
    local newSkin = Skin()
    addmodifier obj newSkin
    
    newSkin.filter_cross_sections = off
    newSkin.filter_vertices = on
    newSkin.showNoEnvelopes = on
    newSkin.wt_showGlobal = on
    newSkin.wt_activeVertexSet = 1
    newSkin.wt_showAffectedBones = on
    
    for i = 1 to NameArray.count do
    (
        addbone newSkin (getNodeByName NameArray[i]) 1
    )
    
    completeredraw()

    --GO TO THE BEGINING OF THE STREAM STREAM, AND COLLECT THE BONES
    seek createms 0
    
    --LOAD THE WEIGHTS
    while not eof createms do
    (
        boneIDarray = #()
        
        a = execute(readline createms)
        b = execute(readline createms)
        c = execute(readline createms)
        
        local bCount = c.count
        
        for i = 1 to bCount do
        (
            local BoneID = findItem NameArray c[i]
            
            if BoneID != 0 do
            (
                append boneIDarray BoneID
            )
        )
            
        ReplaceVertexWeights newSkin a boneIDarray b
    )		
)

fn ToggleAD state =
(
    for o in objects do
        (
            --collect all modifiers in the current object
            theMods = for m in o.modifiers collect m
            --loop from top to bottom and if the modifier is a Skin     
            for m in theMods where classof m == Skin do 
            (        
                try
                (
                    m.always_deform = state
                )
                catch(format "*** % ***\n" (getCurrentException()))
            )	
        )
        completeRedraw()
)

fn ClearUVW03 o i = 
(
    local clear0 = MapChannelDelete()
    local clear3 = MapChannelDelete()
    addModifier o clear3 before:i
    clear3.mapID = 3
    addModifier o clear0 before:i
    clear0.mapID = 0
)

fn XFormReset o =
(
    ResetXForm o
    if (classof o.modifiers[1]) != Skin do try(SkinOnTop o)
        catch(format "\%n" (getcurrentexception())) 
    ForceCompleteRedraw()
)

--QoL UI code start
if(getINISetting "$scripts\Resources\Settings.ini" "UI Mode" "WideUI" as BooleanClass) then
(
    rollout QoL "QoL Tool" --Wide UI
    (
        button btnMoveSkin "Skin To Top" align:#center width:100 across:2 offset:[0,5]
        label ADLabel "Always Deform:" offset:[0,5]

        button btnConvert "Mesh To Poly" align:#center width:100 across:3 offset:[22,0]
        button btnADOn "On" width:30 offset:[45,-11]
        button btnADOff "Off" width:30 offset:[-5,-11]

        button btnClrUVW "Clear UVW 0/3" align:#center width:100 across:3 offset:[22,0]
        button btnCombineSkin "Combine Skin"  width:100 offset:[65,-11]
        button btnBackface "Add Backfaces" width:100 offset:[-28,15]

        button btnXForm "Reset XForm" width:100 align:#left offset:[15,-15]

        on btnXForm pressed do 
        (            
            selObjs = getCurrentSelection()			
            for o in selObjs do
            (
                XFormReset o                              
            )            
        )
        
        on btnBackface pressed do
        (
            try(createBackface())
            catch(format "*** % ***\n" (getCurrentException()))
        )
	
        on btnCombineSkin pressed do
        (
            try(combineSkin())
            catch(format "\%n" (getcurrentexception()))
        )

        on btnADOff pressed do
        (			
            ToggleAD(false)
            print "Unchecked Always Deform."
        )
        
        on btnADOn pressed do
        (			
            ToggleAD(true)
            print "Checked Always Deform."
        )
        
        on btnMoveSkin pressed do
        (			
            selObjs = getCurrentSelection()			
            for o in selObjs do
            (
                if (classof o.modifiers[1]) != Skin do try(SkinOnTop o)
                catch(format "\%n" (getcurrentexception()))
            )	
        )
        
        on btnConvert pressed do 
        (			
            selObjs = getCurrentSelection()			
            for o in selObjs where classof o == Editable_mesh do
            (
                try(MeshToPoly o)
                catch(format "*** % ***\n" (getCurrentException()))					
            )
        )
        
        on btnClrUVW pressed do
        (
            selObjs = getCurrentSelection()			
            for o in selObjs do
            (
                for i = 1 to o.modifiers.count where classof o.modifiers[i] == Skin do
                (
                    clearUVW03 o i	
                )		
            )
        )
        
        on btnHelpQoL pressed do messagebox "Skin To Top: Moves all Skin modifiers to the top of the stack. \n\nMesh To Poly: Converts any selected Editable Mesh to Editable Poly.\n\nClear UVW 0/3: Adds two UVW Mapping Clear modifiers set to channels 0 and 3." Title:"Help" beep:false

    )
) else (
    rollout QoL "QoL Tool" --Thin UI
    (
        button btnMoveSkin "Skin To Top" align:#center width:100
        button btnConvert "Mesh To Poly" align:#center width:100
        button btnClrUVW "Clear UVW 0/3" align:#center width:100      
        button btnXForm "Reset XForm" width:100 align:#center

        label ADLabel "Always Deform:"
        button btnADOn "On" width:30 across:2 offset:[8,0]
        button btnADOff "Off" width:30 offset:[-8,0]        

        button btnCombineSkin "Combine Skin" align:#center width:100
        button btnBackface "Add Backfaces" width:100 align:#center        

        on btnXForm pressed do 
        (            
            selObjs = getCurrentSelection()			
            for o in selObjs do
            (
                XFormReset o                              
            )            
        )
        
        on btnBackface pressed do
        (
            try(createBackface())
            catch(format "*** % ***\n" (getCurrentException()))
        )
	
        on btnCombineSkin pressed do
        (
            try(combineSkin())
            catch(format "\%n" (getcurrentexception()))
        )
        
        on btnADOff pressed do
        (			
            ToggleAD(false)
            print "Unchecked Always Deform."
        )
        
        on btnADOn pressed do
        (			
            ToggleAD(true)
            print "Checked Always Deform."
        )
        
        on btnMoveSkin pressed do
        (			
            selObjs = getCurrentSelection()			
            for o in selObjs do
            (
                if (classof o.modifiers[1]) != Skin do try(SkinOnTop o)
                catch(format "\%n" (getcurrentexception()))
            )	
        )
        
        on btnConvert pressed do 
        (			
            selObjs = getCurrentSelection()			
            for o in selObjs where classof o == Editable_mesh do
            (
                try(MeshToPoly o)
                catch(format "*** % ***\n" (getCurrentException()))					
            )
        )
        
        on btnClrUVW pressed do
        (
            selObjs = getCurrentSelection()			
            for o in selObjs do
            (
                for i = 1 to o.modifiers.count where classof o.modifiers[i] == Skin do
                (
                    clearUVW03 o i	
                )		
            )
        )
    )
)


